<?php
/* Copyright (C) 2005-2014 Laurent Destailleur	<eldy@users.sourceforge.net>
 * Copyright (C) 2005-2014 Regis Houssin		<regis.houssin@capnetworks.com>
 * Copyright (C) 2014      Marcos García		<marcosgdf@gmail.com>
 * Copyright (C) 2015      Bahfir Abbes        <bafbes@gmail.com>
 * Copyright (C) 2017      Nicolas ZABOURI        <info@inovea-conseil.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
/**
 *  \file       htdocs/core/triggers/interface_90_all_Demo.class.php
 *  \ingroup    core
 *  \brief      Fichier de demo de personalisation des actions du workflow
 *  \remarks    Son propre fichier d'actions peut etre cree par recopie de celui-ci:
 *              - Le nom du fichier doit etre: interface_99_modMymodule_Mytrigger.class.php
 *                                           ou: interface_99_all_Mytrigger.class.php
 *              - Le fichier doit rester stocke dans core/triggers
 *              - Le nom de la classe doit etre InterfaceMytrigger
 *              - Le nom de la propriete name doit etre Mytrigger
 */
require_once DOL_DOCUMENT_ROOT . '/core/triggers/dolibarrtriggers.class.php';

/**
 *  Class of triggers for demo module
 */
class InterfaceProducts extends DolibarrTriggers
{
    public $family = 'other';
    public $picto = 'technic';
    public $description = "Triggers of this module are empty functions. They have no effect. They are provided for tutorial purpose only.";
    public $version = self::VERSION_DOLIBARR;

    /**
     * Function which update the informations of a line of the document (Propal)
     *
     * @param int $tx_tva
     * @param int $ht
     * @param int $ttc
     * @param string $tva
     * @param int $id
     * @param string $tableName
     * @global object $conf
     * @global type $db
     */
    private function updateLine($tx_tva, $ht, $ttc, $tva, $id, $tableName, $object = null)
    {
        global $conf, $db;
        $sql = "UPDATE " . MAIN_DB_PREFIX . $tableName . " SET ";
        $sql .= " tva_tx =" . $tx_tva;
        if ($object != null) {
            $sql .= ", subprice = " . price2num($object->subprice);
        }
        $sql .= ", total_ht=" . price2num($ht);
        $sql .= ", total_ttc=" . price2num($ttc);
        $sql .= ", total_tva=" . price2num($tva);
        $sql .= " WHERE rowid=" . $id;
        $resql = $db->query($sql);
        if (!$resql) {
            dol_print_error($db);
            exit;
        }
    }

    /**
     * Function which update the informations of the document (Propal)
     *
     * @param int $ht Total ht for the document
     * @param int $ttc Total ttc for the document
     * @param int $tva Total of vat for the document
     * @param int $id Id of the object to update
     * @global object $db Object of the database
     */
    private function updateTotalPropal($ht, $ttc, $tva, $id)
    {
        global $db;

        $total_ttc = "total_ttc";
        $total_ht = "total_ht";
        $total_tva = "total_tva";
        if((int)DOL_VERSION < 14){
            $total_tva = "tva";
            $total_ttc = "total";
        }

        $sql = "UPDATE " . MAIN_DB_PREFIX . "propal SET $total_ht = " . $ht . " , $total_ttc =" . $ttc . " , $total_tva = " . $tva . " ";
        $sql .= " WHERE rowid=" . $id . " ";

        $resql = $db->query($sql);
        if (!$resql) {
            dol_print_error($db);
            exit;
        }
    }

    /**
     * Function which update the informations of the document (Order)
     *
     * @param int $ht Total ht for the document
     * @param int $ttc Total ttc for the document
     * @param int $tva Total of vat for the document
     * @param int $id Id of the object to update
     * @global object $db Object of the database
     */
    private function updateTotalOrder($ht, $ttc, $tva, $id)
    {
        global $db;

        $total_ttc = "total_ttc";
        $total_ht = "total_ht";
        $total_tva = "total_tva";
        if((int)DOL_VERSION <= 13){
            $total_tva = "tva";
        }

        $sql = "UPDATE " . MAIN_DB_PREFIX . "commande SET $total_ht = " . $ht . " , $total_ttc = " . $ttc . " , $total_tva =" . $tva . " ";
        $sql .= " WHERE rowid=" . $id . " ";

        $resql = $db->query($sql);
        if (!$resql) {
            dol_print_error($db);
            exit;
        }
    }

    /**
     * Function which update the informations of the document (Invoice)
     *
     * @param int $ht Total ht for the document
     * @param int $ttc Total ttc for the document
     * @param int $tva Total of vat for the document
     * @param int $id Id of the object to update
     * @global object $db Object of the database
     */
    private function updateTotalInvoice($ht, $ttc, $tva, $id)
    {
        global $db;

        $total_ttc = "total_ttc";
        $total_ht = "total_ht";
        $total_tva = "total_tva";
        if((int)DOL_VERSION < 14){
            $total_ht = "total";
            $total_tva = "tva";
        }

        $sql = "UPDATE " . MAIN_DB_PREFIX . "facture SET $total_ht = " . $ht . " , $total_ttc = " . $ttc . " , $total_tva =" . $tva . " ";
//        print '<pre>' . print_r($sql, 1) . '</pre>';

        $sql .= " WHERE rowid=" . $id . " ";
        $resql = $db->query($sql);
        if (!$resql) {
            dol_print_error($db);
            exit;
        }
    }

    /**
     * Calculation of vat margin for the document choosen ($object)
     *
     * @param object $object Object to update (propal, order, invoice)
     * @param string $tableNameForElementLine Name of the table to update (SQL)
     * @param string $functionName Name of function to update the database which the object is linked
     * @global object $conf Object of conf
     */
    private function calculationOfVATMargin($object, $tableNameForElementLine, $functionName, $element = null, $fk_element = null, $isDelete = 0)
    {
        global $conf, $db;
        if ($object->lines != null) {
            foreach ($object->lines as $l) {
                if ($l->product_type != 9) {
                    $totTTC = $l->total_ht;
                    $margeTTC = $totTTC - ($l->qty * $l->pa_ht);
                    if ($margeTTC > 0) {
                        switch ($l->tva_tx) {
                            case '5.500':
                                $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_5;
                                $tva = $margeHT * 0.055;
                                break;
                            case '10.000':
                                $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_10;
                                $tva = $margeHT * 0.10;
                                break;
                            case '20.000':
                                $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_20;
                                $tva = $margeHT * 0.20;
                                break;
                            default:
                                $tva = 0;
                                break;
                        }
                        $this->updateLine($l->tva_tx, price2num($totTTC - $tva, 'MU'), price2num($l->total_ht, 'MU'), price2num($tva, 'MU'), $l->rowid, $tableNameForElementLine);
                        $sumTTC += $l->total_ht;
                        $sumHT += $l->total_ttc - $tva;
                        $sumTVA += $tva;
                    } else {
                        $sumTTC += $l->total_ttc;
                        $sumHT += $l->total_ht;
                        $sumTVA += $l->total_tva;
                    }
                }
            }
            $this->$functionName(price2num($sumHT, 'MT'), price2num($sumTTC, 'MT'), price2num($sumTVA, 'MT'), $object->id);
        } else if ($element != null) {
            if ($object->product_type != 9) {
                $totTTC = $object->total_ht;
                $margeTTC = $totTTC - ($object->qty * $object->pa_ht);
                if ($margeTTC > 0) {
                    switch ($object->tva_tx) {
                        case '5.500':
                            $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_5;
                            $tva = $margeHT * 0.055;
                            break;
                        case '10.000':
                            $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_10;
                            $tva = $margeHT * 0.10;
                            break;
                        case '20.000':
                            $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_20;
                            $tva = $margeHT * 0.20;
                            break;
                        default:
                            $tva = 0;
                            break;
                    }
                    $this->updateLine($object->tva_tx, price2num($totTTC - $tva, 'MU'), price2num($object->total_ht, 'MU'), price2num($tva, 'MU'), $object->rowid, $tableNameForElementLine, $object);
                }
            }
            $p = new $element($db);
            $p->fetch($object->$fk_element);
            foreach ($p->lines as $l) {
                if ($l->product_type != 9) {
                    $totTTC = $l->total_ttc;
                    $margeTTC = $totTTC - ($l->qty * $l->pa_ht);
                    if ($margeTTC > 0) {
                        switch ($l->tva_tx) {
                            case '5.500':
                                $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_5;
                                $tva = $margeHT * 0.055;
                                break;
                            case '10.000':
                                $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_10;
                                $tva = $margeHT * 0.10;
                                break;
                            case '20.000':
                                $margeHT = $margeTTC * $conf->global->VATMARGIN_ON_TVA_20;
                                $tva = $margeHT * 0.20;
                                break;
                            default:
                                $tva = 0;
                                break;
                        }
                        if (!$isDelete || ($isDelete && ($l->id != $object->id))) {
                            $sumTTC += $l->total_ttc;
                            $sumHT += $l->total_ttc - $tva;
                            $sumTVA += $tva;
                        }
                    } else {
                        if (!$isDelete || ($isDelete && ($l->id != $object->id))) {
                            $sumTTC += $l->total_ttc;
                            $sumHT += $l->total_ht;
                            $sumTVA += $l->total_tva;
                        }
                    }
                }
            }
            $this->$functionName(price2num($sumHT, 'MT'), price2num($sumTTC, 'MT'), price2num($sumTVA, 'MT'), $p->id);
        }
    }

    /**
     * Update the price when vat margin is not used
     *
     * @param object $object Object to update (propal, order, invoice)
     * @param string $type Type of document to update (propal, order, invoice)
     * @global object $conf Object of conf
     */
    private function updatePriceWhenVATMarginNotActivate($object, $type)
    {
        global $conf;
        $constant = 'MODULE_DISALLOW_UPDATE_PRICE_' . $type;
        $modWhichUseConst = explode(',', $conf->global->$constant);
        if (count($modWhichUseConst) == 1 && $modWhichUseConst[0] == 'vatmargin') {
            $conf->global->$constant = '';
            //print '<pre>' . print_r($conf, 1) . '</pre>';
            $object->update_price(1);
            $conf->global->$constant = 'vatmargin';
        } else {
            $temp = '';
            foreach ($modWhichUseConst as $mod) {
                if ($mod != 'vatmargin' && trim($mod) != null) {
                    if ($temp != '') {
                        $temp .= ',';
                    }
                    $temp .= $mod;
                }
            }
            $conf->global->$constant = $temp;
            $object->update_price(1);
            $conf->global->$constant .= 'vatmargin';
        }
    }

    /**
     * Function called when a Dolibarrr business event is done.
     * All functions "runTrigger" are triggered if file is inside directory htdocs/core/triggers or htdocs/module/code/triggers (and declared)
     *
     * @param string $action Event action code
     * @param Object $object Object concerned. Some context information may also be provided into array property object->context.
     * @param User $user Object user
     * @param Translate $langs Object langs
     * @param conf $conf Object conf
     * @return int <0 if KO, 0 if no triggered ran, >0 if OK
     */
    public function runTrigger($action, $object, User $user, Translate $langs, Conf $conf)
    {
        global $db, $conf;

        require_once DOL_DOCUMENT_ROOT . '/comm/propal/class/propal.class.php';
        require_once DOL_DOCUMENT_ROOT . '/commande/class/commande.class.php';
        require_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';
        require_once DOL_DOCUMENT_ROOT . '/product/class/product.class.php';

        if ($conf->global->MAIN_MODULE_VATMARGIN) {
            switch ($action) {
                case 'LINEPROPAL_INSERT':
                    $p = new Propal($db);
                    $p->fetch($object->fk_propal);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
//                        if ($object->fk_product != null && $object->tva_tx != 0) {
//                            $product = new Product($db);
//                            $res = $product->fetch($object->fk_product);
//                            if ($res > 0) {
//                                if ($object->tva_tx != $product->tva_tx) {
//                                    $object->total_ht = (($product->price + ($product->price * ($object->tva_tx / 100))) * $object->qty);
//                                    $object->subprice = ($product->price + ($product->price * ($object->tva_tx / 100)));
//                                } else {
//                                    $object->total_ht = $product->price_ttc * $object->qty;
//                                    $object->subprice = $product->price_ttc;
//                                }
//                            }
//                        }
                        $this->calculationOfVATMargin($object, 'propaldet', 'updateTotalPropal', 'Propal', 'fk_propal');
                    }
                case 'LINEPROPAL_UPDATE':
                case 'LINEPROPAL_MODIFY':
                    $p = new Propal($db);
                    $p->fetch($object->fk_propal);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
//                        $product = new Product($db);
//                        $res = $product->fetch($object->fk_product);
//                        if ($res > 0) {
//                            if ($object->tva_tx != $product->tva_tx) {
//                                $object->total_ht = (($product->price + ($product->price * ($object->tva_tx / 100))) * $object->qty);
//                                $object->subprice = ($product->price + ($product->price * ($object->tva_tx / 100)));
//                            } else {
//                                $object->total_ht = $product->price_ttc * $object->qty;
//                                $object->subprice = $product->price_ttc;
//                            }
//                        }
                        $this->calculationOfVATMargin($object, 'propaldet', 'updateTotalPropal', 'Propal', 'fk_propal');
                    }
//                    else {
//                        $this->updatePriceWhenVATMarginNotActivate($p, 'PROPOSAL');
//                    }
                    break;
                case 'LINEPROPAL_DELETE':
                    $p = new Propal($db);
                    $p->fetch($object->fk_propal);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
                        $this->calculationOfVATMargin($object, 'propaldet', 'updateTotalPropal', 'Propal', 'fk_propal', 1);
                    }
//                    else {
//                        $this->updatePriceWhenVATMarginNotActivate($p, 'PROPOSAL');
//                    }
                    break;
                case 'LINEORDER_INSERT':
                    $p = new Commande($db);
                    $p->fetch($object->fk_commande);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
                        $this->calculationOfVATMargin($object, 'commandedet', 'updateTotalOrder', 'Commande', 'fk_commande');
                    }
                case 'LINEORDER_UPDATE':
                case 'LINEORDER_MODIFY':
                    $p = new Commande($db);
                    $p->fetch($object->fk_commande);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
//                        $product = new Product($db);
//                        $res = $product->fetch($object->fk_product);
//                        if ($res > 0) {
//                            if ($object->tva_tx != $product->tva_tx) {
//                                $object->total_ht = (($product->price + ($product->price * ($object->tva_tx / 100))) * $object->qty);
//                                $object->subprice = ($product->price + ($product->price * ($object->tva_tx / 100)));
//                            } else {
//                                $object->total_ht = $product->price_ttc * $object->qty;
//                                $object->subprice = $product->price_ttc;
//                            }
//                        }
                        $this->calculationOfVATMargin($object, 'commandedet', 'updateTotalOrder', 'Commande', 'fk_commande');
                    }
//                    else {
//                        $this->updatePriceWhenVATMarginNotActivate($p, 'ORDER');
//                    }
                    break;
                case 'LINEORDER_DELETE':
                    $p = new Commande($db);
                    $p->fetch($object->fk_commande);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
                        $this->calculationOfVATMargin($object, 'commandedet', 'updateTotalOrder', 'Commande', 'fk_commande', 1);
                    }
//                    else {
//                        $this->updatePriceWhenVATMarginNotActivate($p, 'ORDER');
//                    }
                    break;
                case 'LINEBILL_INSERT':
                    $p = new Facture($db);
                    $p->fetch($object->fk_facture);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
//                        if ($object->fk_product != null && $object->tva_tx != 0) {
//                            $product = new Product($db);
//                            $res = $product->fetch($object->fk_product);
//                            if ($res > 0) {
//                                if ($object->tva_tx != $product->tva_tx) {
//                                    $object->total_ht = (($product->price + ($product->price * ($object->tva_tx / 100))) * $object->qty);
//                                    $object->subprice = ($product->price + ($product->price * ($object->tva_tx / 100)));
//                                } else {
//                                    $object->total_ht = $product->price_ttc * $object->qty;
//                                    $object->subprice = $product->price_ttc;
//                                }
//                            }
//                        }
                        $this->calculationOfVATMargin($object, 'facturedet', 'updateTotalInvoice', 'Facture', 'fk_facture');
                    }
                case 'LINEBILL_MODIFY':
                case 'LINEBILL_UPDATE':
                    $p = new Facture($db);
                    $p->fetch($object->fk_facture);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
                        $product = new Product($db);
//                        $res = $product->fetch($object->fk_product);
//                        if ($res > 0) {
//                            if ($object->tva_tx != $product->tva_tx) {
//                                $object->total_ht = (($product->price + ($product->price * ($object->tva_tx / 100))) * $object->qty);
//                                $object->subprice = ($product->price + ($product->price * ($object->tva_tx / 100)));
//                            } else {
//                                $object->total_ht = $product->price_ttc * $object->qty;
//                                $object->subprice = $product->price_ttc;
//                            }
//                        }
                        $this->calculationOfVATMargin($object, 'facturedet', 'updateTotalInvoice', 'Facture', 'fk_facture');
                    }
//                    else {
//                        $this->updatePriceWhenVATMarginNotActivate($p, 'INVOICE');
//                    }
                    break;
                case 'LINEBILL_DELETE':
                    $p = new Facture($db);
                    $p->fetch($object->fk_facture);
                    $activ = $p->array_options['options_vatmargin'];
                    if ($activ == 2) {
                        $this->calculationOfVATMargin($object, 'facturedet', 'updateTotalInvoice', 'Facture', 'fk_facture', 1);
                    }
//                    else {
//                        $this->updatePriceWhenVATMarginNotActivate($p, 'INVOICE');
//                    }
                    break;
            }
        }
        return 0;
        /*
          case 'LINEPRESELECTION_INSERT':
          case 'LINEPRESELECTION_UPDATE':
          case 'LINEPRESELECTION_DELETE':
          require_once DOL_DOCUMENT_ROOT . '/preselection/class/preselection.class.php';
          $p = new Preselection($db);
          $p->fetch($object->fk_preselection);
          $activ = $p->array_options['options_vatmargin'];
          if ($activ == 2) {
          foreach ($p->lines as $l) {
          if ($l->product_type != 9) {
          if ($l->qty > 0) {
          $sum += $l->total_ht;
          $marge += $l->total_ht - ($l->qty * $l->pa_ht);
          // $tva += $l->total_tva;
          }
          }
          }
          $vat = $conf->global->VATMARGIN / 100;
          $vatmarge = $marge * $vat;
          $tva = $vatmarge;
          $ttc = $sum + $tva;
          $sql = "UPDATE " . MAIN_DB_PREFIX . "preselection SET total_ht = " . $sum . " ,total =" . $ttc . " , tva =" . $tva . " ";
          $sql .= " WHERE rowid=" . $object->fk_preselection . " ";
          $db->query($sql);
          } else {
          $temp = $conf->global->MODULE_DISALLOW_UPDATE_PRICE_PRESELECTION;
          $conf->global->MODULE_DISALLOW_UPDATE_PRICE_PRESELECTION = '';
          $p->update_price(1);
          $conf->global->MODULE_DISALLOW_UPDATE_PRICE_PRESELECTION = $temp;
          }
          break;
         */
    }
}